#-*- coding:utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Allen Lowe <lallenlowe@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

"""
    This module provides the classes that make up the data structures for the
    Dexter address book. It utilizes the Storm ORM for interfacing with the db.
    It also provides the functions for initializing a new database.
"""

import storm.locals as storm
import glib
import gio

def get_dexter_data_dir():
    global USER_DIR
    data_dir = gio.File(USER_DIR + "/dexter")
    if not data_dir.query_exists():
        data_dir.make_directory()
    return data_dir.get_path()
    
def get_image_dir():
    global DATA_DIR
    image_dir = gio.File(DATA_DIR + "/images")
    if not image_dir.query_exists():
        image_dir.make_directory()
    return image_dir.get_path()

USER_DIR = glib.get_user_data_dir()
DATA_DIR = get_dexter_data_dir()
SETUP_COMPLETE = DATA_DIR + "/.setup_complete" # no longer used I believe
IMAGE_DIR = get_image_dir()


class Contact(storm.Storm):
    """a class for storing/accessing a contact"""
    __storm_table__ = "contact"
    id = storm.Int(primary=True)
    photo = storm.Unicode()
    first_name = storm.Unicode()
    middle_name = storm.Unicode()
    last_name = storm.Unicode()
    full_name = storm.Unicode()
    organization = storm.Unicode()
    birthday = storm.Date()
    phones = storm.ReferenceSet(id, "Phone.contact_id")
    emails = storm.ReferenceSet(id, "Email.contact_id")
    addresses = storm.ReferenceSet(id, "Address.contact_id")
    imnicks = storm.ReferenceSet(id, "IMNick.contact_id")
    notes = storm.ReferenceSet(id, "Note.contact_id")

    def __init__(self, first_name):
        if first_name:
            self.first_name = first_name

    def create_full_name(self):
        fullname = self.first_name
        if self.middle_name:
            fullname += (" " + self.middle_name)
        if self.last_name:
            fullname += (" " + self.last_name)
        self.full_name = fullname


class Address(storm.Storm):
    """a class for storing/accessing a postal address for a contact"""
    __storm_table__ = "address"
    id = storm.Int(primary=True)
    street = storm.Unicode()
    city = storm.Unicode()
    state = storm.Unicode()
    zip = storm.Unicode()
    address_type = storm.Unicode()
    contact_id = storm.Int()
    contact = storm.Reference(contact_id, "Contact.id")

    def __init__(self, street=None, city=None, state=None, zip_=None,
                 address_type=None):
        if street:
            self.street = unicode("".join(street))
        if city:
            self.city = city
        if state:
            self.state = state
        if zip_:
            self.zip = zip_
        if address_type:
            self.address_type = address_type


class Phone(storm.Storm):
    """a class for storing/accessing phone numbers for a contact"""
    __storm_table__ = "phone"
    id = storm.Int(primary=True)
    number = storm.Unicode()
    phone_type = storm.Unicode()
    contact_id = storm.Int()
    contact = storm.Reference(contact_id, "Contact.id")

    def __init__(self, number, phone_type=None):
        self.number = number
        if not phone_type:
            self.phone_type = u"Mobile"
        else:
            self.phone_type = phone_type


class Email(storm.Storm):
    """a class for storing/accessing emails for a contact"""
    __storm_table__ = "email"
    id = storm.Int(primary=True)
    address = storm.Unicode()
    email_type = storm.Unicode()
    contact_id = storm.Int()
    contact = storm.Reference(contact_id, "Contact.id")

    def __init__(self, address, email_type=None):
        self.address = address
        if not email_type:
            self.email_type = u"Home"
        else:
            self.email_type = email_type


class IMNick(storm.Storm):
    """a class for storing/accessing im nicks for a contact"""
    __storm_table__ = "imnick"
    id = storm.Int(primary=True)
    nick = storm.Unicode()
    nick_type = storm.Unicode()
    contact_id = storm.Int()
    contact = storm.Reference(contact_id, "Contact.id")

    def __init__(self, nick, nick_type=None):
        self.nick = nick
        if not nick_type:
            self.nick_type = u"IM"
        else:
            self.nick_type = nick_type


class Note(storm.Storm):
    """a class for storing/accessing notes for a contact"""
    __storm_table__ = "note"
    id = storm.Int(primary=True)
    text = storm.Unicode()
    contact_id = storm.Int()
    contact = storm.Reference(contact_id, "Contact.id")

    def __init__(self, text):
        self.text = text


def setup_db(data_dir):
    """create a database file if it doesn't already exist, then create a store"""
    database = storm.create_database("sqlite:" + data_dir + "/dexterdb")
    store = storm.Store(database)
    return store


def create_tables(store):
    try:
        #create the contact table
        store.execute("CREATE TABLE contact "
                      "(id INTEGER PRIMARY KEY, photo VARCHAR, first_name VARCHAR, \
                      middle_name VARCHAR, last_name VARCHAR, full_name VARCHAR, \
                      organization VARCHAR, birthday VARCHAR)")
        #create the phone table
        store.execute("CREATE TABLE phone "
                      "(id INTEGER PRIMARY KEY, number VARCHAR, phone_type VARCHAR,\
                      contact_id INTEGER)")
        #create the email table
        store.execute("CREATE TABLE email "
                      "(id INTEGER PRIMARY KEY, address VARCHAR, email_type VARCHAR,\
                      contact_id INTEGER)")
        #create the address table
        store.execute("CREATE TABLE address "
                      "(id INTEGER PRIMARY KEY, street VARCHAR, city VARCHAR,\
                        state VARCHAR, zip VARCHAR, address_type VARCHAR,\
                        contact_id INTEGER)")
        #create the imnick table
        store.execute("CREATE TABLE imnick "
                      "(id INTEGER PRIMARY KEY, nick VARCHAR, nick_type VARCHAR,\
                      contact_id INTEGER)")
        #create the note table
        store.execute("CREATE TABLE note "
                      "(id INTEGER PRIMARY KEY, text VARCHAR, contact_id INTEGER)")
    except:
        #tables already created, moving on...
        pass
    #save all changes
    setup_complete()  # no longer used I believe
    store.flush()
    store.commit()


# keep each change/update in it's own try block,
# so that users can upgrade successfully from any previous version of Dexter
def update_tables(store):
    """run any necessary schema changes or db updates"""
    try:
        store.execute("ALTER TABLE contact ADD COLUMN middle_name VARCHAR")
    except:
        pass
    try:
        store.execute("ALTER TABLE contact ADD COLUMN full_name VARCHAR")
    except:
        pass
    try:
        #create the imnick table
        store.execute("CREATE TABLE imnick "
                      "(id INTEGER PRIMARY KEY, nick VARCHAR, nick_type VARCHAR,\
                      contact_id INTEGER)")
    except:
        pass
    contacts = store.find(Contact)
    for contact in contacts:
        if not contact.full_name:
            contact.create_full_name()
    store.flush()
    store.commit()

# no longer used I believe
def setup_complete():
    global SETUP_COMPLETE
    completed_file = open(SETUP_COMPLETE, "w")
    completed_file.write("True")
    completed_file.close()

if __name__ == "__main__":
    store = setup_db(DATA_DIR)
    create_tables(store)
